@@ -1,26 +1,12 @@
 # argumentor - A simple, copylefted, lightweight library to work with command-line arguments in Python
 # Copyright (C) 2021 Twann <twann@ctemplar.com>
 
-# This program is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <https://www.gnu.org/licenses/>.
-
-
 import sys
 import re
 from .exceptions import ExecNameError, OperationExistsError, OptionExistsError, ArgumentValueError, GroupExistsError, \
-    GroupNotExistsError, InvalidOptionError
+    GroupNotExistsError, InvalidOptionError, InvalidOperationError
 
-__version__ = '0.0.3'
+__version__ = '0.0.4'
 
 
 class Arguments:
@@ -100,59 +86,79 @@ class Arguments:
         else:
             output += f'usage: {self.exec_name} [arg(s)]\n\n'
 
-        output += '<operations>\n'
+        if self.operations != {}:
+            output += '<operations>\n'
 
-        for group in self.operation_groups.keys():
-            if self.show_operation_groups_on_help_page:
-                output += f'* {group}\n'
-            for operation in self.__get_operations_from_group(group):
-                line = ''
-                if self.operations[operation]['short_operation'] is not None:
-                    line += f'  {self.operations[operation]["short_operation"]}'
-                line += f'  {operation}'
-                if len(line) > 15:
-                    line += f'\t\t\t{self.operations[operation]["description"]}\n'
-                else:
-                    line += f'\t\t\t\t{self.operations[operation]["description"]}\n'
-                output += line
-        for operation in self.operations:
-            if self.operations[operation]['group'] is None:
-                line = ''
-                if self.operations[operation]['short_operation'] is not None:
-                    line += f'  {self.operations[operation]["short_operation"]}'
-                line += f'  {operation}'
-                if len(line) > 15:
-                    line += f'\t\t\t{self.operations[operation]["description"]}\n'
-                else:
-                    line += f'\t\t\t\t{self.operations[operation]["description"]}\n'
-                output += line
+            for operation in self.operations:
+                if self.operations[operation]['group'] is None:
+                    line = ''
+                    if self.operations[operation]['short_operation'] is not None:
+                        line += f'  {self.operations[operation]["short_operation"]}'
+                    line += f'  {operation}'
+                    if self.operations[operation]['description'] is not None:
+                        if len(line) > 15:
+                            line += f'\t\t\t{self.operations[operation]["description"]}\n'
+                        else:
+                            line += f'\t\t\t\t{self.operations[operation]["description"]}\n'
+                    else:
+                        line += '\n'
+                    output += line
+            for group in self.operation_groups.keys():
+                if self.show_operation_groups_on_help_page:
+                    if self.operation_groups[group]["name"]:
+                        output += f'* {self.operation_groups[group]["name"]}\n'
+                    else:
+                        output += f'* {group}\n'
+                for operation in self.__get_operations_from_group(group):
+                    line = ''
+                    if self.operations[operation]['short_operation'] is not None:
+                        line += f'  {self.operations[operation]["short_operation"]}'
+                    line += f'  {operation}'
+                    if self.operations[operation]['description'] is not None:
+                        if len(line) > 15:
+                            line += f'\t\t\t{self.operations[operation]["description"]}\n'
+                        else:
+                            line += f'\t\t\t\t{self.operations[operation]["description"]}\n'
+                    else:
+                        line += '\n'
+                    output += line
 
-        output += '\n<options>\n'
+        if self.options != {}:
+            output += '\n<options>\n'
 
-        for group in self.option_groups.keys():
-            if self.show_option_groups_on_help_page:
-                output += f'* {group}\n'
-            for option in self.__get_options_from_group(group):
-                line = ''
-                if self.options[option]['short_option'] is not None:
-                    line += f'  {self.options[option]["short_option"]}'
-                line += f'  {option}'
-                if len(line) > 15:
-                    line += f'\t\t\t{self.options[option]["description"]}\n'
-                else:
-                    line += f'\t\t\t\t{self.options[option]["description"]}\n'
-                output += line
-        for option in self.options:
-            if self.options[option]['group'] is None:
-                line = ''
-                if self.options[option]['short_operation'] is not None:
-                    line += f'  {self.options[option]["short_option"]}'
-                line += f'  {option}'
-                if len(line) > 15:
-                    line += f'\t\t\t{self.options[option]["description"]}\n'
-                else:
-                    line += f'\t\t\t\t{self.options[option]["description"]}\n'
-                output += line
+            for option in self.options:
+                if self.options[option]['group'] is None:
+                    line = ''
+                    if self.options[option]['short_option'] is not None:
+                        line += f'  {self.options[option]["short_option"]}'
+                    line += f'  {option}'
+                    if self.options[option]['description'] is not None:
+                        if len(line) > 15:
+                            line += f'\t\t\t{self.options[option]["description"]}\n'
+                        else:
+                            line += f'\t\t\t\t{self.options[option]["description"]}\n'
+                    else:
+                        line += '\n'
+                    output += line
+            for group in self.option_groups.keys():
+                if self.show_option_groups_on_help_page:
+                    if self.option_groups[group]["name"]:
+                        output += f'* {self.option_groups[group]["name"]}\n'
+                    else:
+                        output += f'* {group}\n'
+                for option in self.__get_options_from_group(group):
+                    line = ''
+                    if self.options[option]['short_option'] is not None:
+                        line += f'  {self.options[option]["short_option"]}'
+                    line += f'  {option}'
+                    if self.options[option]['description'] is not None:
+                        if len(line) > 15:
+                            line += f'\t\t\t{self.options[option]["description"]}\n'
+                        else:
+                            line += f'\t\t\t\t{self.options[option]["description"]}\n'
+                    else:
+                        line += '\n'
+                    output += line
 
         if self.more_info_footer is not None:
             output += f'\n{self.more_info_footer}'
@@ -171,7 +177,8 @@ class Arguments:
             return self.option_groups[group]['options']
 
     def add_operation(self, operation: str, value_type, description: str = None,
-                      short_operation: str = None, required: bool = False, group: str = None) -> None:
+                      short_operation: str = None, required: bool = False, group: str = None,
+                      default_value=None) -> None:
         """
         Add an operation to the list of supported arguments
         :param operation: The operation (ex. '--help')
@@ -180,9 +187,9 @@ class Arguments:
         :param short_operation: Optional. The short version of the operation (ex. '-h')
         :param required: Defaults to False. Choose if the operation is required or not
         :param group: Optional. Specify an operation group
+        :param default_value. Specify the default value to use if the argument is not passed. Defaults to None or False for bool.
         :return:
         """
-
         if operation in self.operations.keys():
             raise OperationExistsError(f'operation "{operation}" already exists')
         elif operation in self.options.keys():
@@ -196,16 +203,23 @@ class Arguments:
         elif group is not None and group not in self.operation_groups:
             raise GroupNotExistsError(f'operation group "{group}" does not exist')
         elif short_operation is not None and len(short_operation) > 1 and not re.match(r'-[A-z0-9]', short_operation):
-            raise InvalidOptionError(f'short operation can only be one-character long when no dash, not "{short_operation}"')
+            raise InvalidOperationError(f'short operation can only be one-character long when no dash, not "{short_operation}"')
         elif short_operation is not None and len(short_operation) > 2:
-            raise InvalidOptionError(f'short operation can contain a dash and a character maximum, not "{short_operation}"')
+            raise InvalidOperationError(f'short operation can contain a dash and a character maximum, not "{short_operation}"')
+        elif short_operation == operation:
+            raise InvalidOperationError('short operation is equal to long operation')
         else:
+            if default_value is None and value_type == bool:
+                default_value = False
+            elif default_value is not None and type(default_value) != value_type:
+                raise ArgumentValueError(f'default value for operation {operation} is from type "{type(default_value)}" instead of "{value_type}"')
             self.operations[operation] = {
                 'value_type': value_type,
                 'description': description,
                 'required': required,
                 'group': group,
-                'short_operation': short_operation
+                'short_operation': short_operation,
+                'default_value': default_value
             }
             if group is not None:
                 self.operation_groups[group]['operations'].append(operation)
@@ -214,22 +228,24 @@ class Arguments:
             if self.generate_help_page:
                 self.help_page = self.__generate_help_page()
 
-    def add_operation_group(self, group_name: str) -> None:
+    def add_operation_group(self, group_name: str, name: str = None) -> None:
         """
         Create an operation group
         :param group_name: The name of the group
+        :param name: Optional. Specify a different name to show on the help page. This can be useful for multi-language support
         :return:
         """
 
         if group_name in self.operation_groups:
             raise GroupExistsError(f'operation group "{group_name}" already exists')
         else:
-            self.operation_groups[group_name] = {'operations': []}
+            self.operation_groups[group_name] = {'operations': [], 'name': name}
 
     def add_option(self, option: str, value_type, description: str = None,
                    short_option: str = None, required: bool = False, available_with_operations: list = None,
                    required_by_operations: list = None, available_with_groups: list = None,
-                   required_by_groups: list = None, group: str = None, inherit_from_group: bool = True) -> None:
+                   required_by_groups: list = None, group: str = None, inherit_from_group: bool = True,
+                   default_value=None) -> None:
         """
         Add an option to the list of supported arguments
         :param option: The option (ex. '--force')
@@ -243,6 +259,7 @@ class Arguments:
         :param required_by_groups: Optional. Every operation in the specified groups will fail without this option
         :param group: Optional. Specify an option group
         :param inherit_from_group: Defaults to True. Inherit requirements and availability from group.
+        :param default_value. Specify the default value to use if the argument is not passed. Defaults to None or False for bool.
         :return:
         """
         if option in self.options.keys():
@@ -264,6 +281,10 @@ class Arguments:
         elif short_option is not None and not re.match(r'-[A-z0-9]', short_option):
             raise InvalidOptionError(f'short option must start with a dash and finish with a character, not "{short_option}"')
         else:
+            if default_value is None and value_type == bool:
+                default_value = False
+            elif default_value is not None and type(default_value) != value_type:
+                raise ArgumentValueError(f'default value for operation {option} is from type "{type(default_value)}" instead of "{value_type}"')
             if group is not None and inherit_from_group:
                 self.options[option] = {
                     'value_type': value_type,
@@ -274,7 +295,8 @@ class Arguments:
                     'required_by_operations': self.option_groups[group]['required_by_operations'],
                     'available_with_groups': self.option_groups[group]['available_with_groups'],
                     'required_by_groups': self.option_groups[group]['required_by_groups'],
-                    'short_option': short_option
+                    'short_option': short_option,
+                    'default_value': default_value
                 }
                 if available_with_operations is not None:
                     self.options[option]['available_with_operations'] = available_with_operations
@@ -294,7 +316,8 @@ class Arguments:
                     'required_by_operations': required_by_operations,
                     'available_with_groups': available_with_groups,
                     'required_by_groups': required_by_groups,
-                    'short_option': short_option
+                    'short_option': short_option,
+                    'default_value': default_value
                 }
             if group is not None:
                 self.option_groups[group]['options'].append(option)
@@ -303,12 +326,13 @@ class Arguments:
             if self.generate_help_page:
                 self.help_page = self.__generate_help_page()
 
-    def add_option_group(self, group_name: str, available_with_operations: list = None,
+    def add_option_group(self, group_name: str, name: str = None, available_with_operations: list = None,
                          required_by_operations: list = None, available_with_groups: list = None,
                          required_by_groups: list = None) -> None:
         """
         Create an option group
         :param group_name: The name of the groups
+        :param name: Optional. Specify a different name to show on the help page. This can be useful for multi-language support
         :param available_with_operations: Optional. Options will only be available with operation in the list
         :param required_by_operations: Optional. Every operation in the list will fail without these options
         :param available_with_groups: Optional. Options will only be available with operation in the specified groups
@@ -324,7 +348,8 @@ class Arguments:
                 'required_by_operations': required_by_operations,
                 'available_with_groups': available_with_groups,
                 'required_by_groups': required_by_groups,
-                'options': []
+                'options': [],
+                'name': name
             }
 
     def __match_operation(self, operation_title: str, real_operation: str, real_argument: str) -> None:
@@ -337,16 +362,11 @@ class Arguments:
                 self.output_operations[operation_title] = True
             elif len(self.arguments) - self.arguments.index(real_argument) > 1:
                 if self.operations[operation_title]['value_type'] == int:
-                    try:
-                        self.output_operations[operation_title] = int(self.arguments[self.arguments.index(real_argument) + 1])
-                    except ValueError:
-                        print(f'Error: operation "{real_operation}" requires an integer')
-                        sys.exit(1)
-                    else:
-                        self.arguments.pop(self.arguments.index(real_argument) + 1)
+                    self.output_operations[operation_title] = None
+                    self.isolate_unmatched = True
                 elif self.operations[operation_title]['value_type'] == str:
-                    self.output_operations[operation_title] = str(self.arguments[self.arguments.index(real_argument) + 1])
-                    self.arguments.pop(self.arguments.index(real_argument) + 1)
+                    self.output_operations[operation_title] = None
+                    self.isolate_unmatched = True
                 elif self.operations[operation_title]['value_type'] == list:
                     self.output_operations[operation_title] = []
                     self.isolate_unmatched = True
@@ -408,6 +428,10 @@ class Arguments:
         :return: operations, options
         """
 
+        # Empty the output arguments to avoid problems if this function is used multiple times
+        self.output_operations = {}
+        self.output_options = {}
+
         for argument in self.arguments:
 
             if argument in self.operations.keys():
@@ -490,20 +514,37 @@ class Arguments:
                         print(f'Error: invalid argument "{character}" in "{argument}"')
                         sys.exit(1)
 
-            elif self.isolate_unmatched and self.active_operation is not None and not re.match(r'-[A-z0-9_.\-]*', argument):
+            elif self.isolate_unmatched and self.active_operation is not None and not re.match(r'-[A-z0-9_.\-]*', argument) \
+                    and self.operations[self.active_operation]['value_type'] == list:
 
-                if not self.operations[self.active_operation]['value_type'] == bool:
-                    self.output_operations[self.active_operation].append(argument)
+                self.output_operations[self.active_operation].append(argument)
+
+            elif self.isolate_unmatched and self.active_operation is not None and not re.match(r'-[A-z0-9_.\-]*', argument) \
+                    and self.operations[self.active_operation]['value_type'] == str and self.output_operations[self.active_operation] is None:
+
+                self.output_operations[self.active_operation] = argument
+
+            elif self.isolate_unmatched and self.active_operation is not None and not re.match(r'-[A-z0-9_.\-]*', argument) \
+                    and self.operations[self.active_operation]['value_type'] == int and self.output_operations[self.active_operation] is None:
+
+                try:
+                    self.output_operations[self.active_operation] = int(argument)
+                except ValueError:
+                    print(f'Error: operation "{argument}" requires an integer')
+                    sys.exit(1)
 
             else:
 
                 print(f'Error: invalid argument "{argument}"')
                 sys.exit(1)
 
-        # Check if all active operations that require a list have at least one value
+        # Check if all active operations that require a list, a str or a int have at least one value
         for active_operation in self.output_operations.keys():
             if self.operations[active_operation]['value_type'] == list and self.output_operations[active_operation] == []:
-                print(f'Error: option "{active_operation}" requires at least one argument')
+                print(f'Error: operation "{active_operation}" requires at least one argument')
+                sys.exit(1)
+            elif self.operations[active_operation]['value_type'] in (str, int) and self.output_operations[active_operation] is None:
+                print(f'Error: operation "{active_operation}" requires at least one argument')
                 sys.exit(1)
 
         # Check if all required operations are active
@@ -522,10 +563,25 @@ class Arguments:
                     if required in self.output_operations.keys():
                         print(f'Error: operation "{required}" requires the option "{option}"')
                         sys.exit(1)
+            if self.options[option]['required_by_groups'] is not None and option not in self.output_options:
+                for required_group in self.options[option]['required_by_groups']:
+                    for operation in self.operations:
+                        if self.operations[operation]['group'] == required_group:
+                            print(f'Error: operation "{operation}" requires the option "{option}"')
+                            sys.exit(1)
 
         # Check if at least one operation is specified
         if self.output_operations == {} and self.one_operation_required:
             print('Error: one operation is required')
             exit(1)
 
+        # Set all the unused arguments to False or None
+        for operation in self.operations:
+            if operation not in self.output_operations:
+                self.output_operations[operation] = self.operations[operation]['default_value']
+
+        for option in self.options:
+            if option not in self.output_options:
+                self.output_options[option] = self.options[option]['default_value']
+
         return self.output_operations, self.output_options
