@@ -1,20 +1,6 @@
 # argumentor - A simple, copylefted, lightweight library to work with command-line arguments in Python
 # Copyright (C) 2021 Twann <twann@ctemplar.com>
 
-# This program is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <https://www.gnu.org/licenses/>.
-
-
 class ExecNameError(IOError):
     def __init__(self, *args):
         super(ExecNameError, self).__init__(*args)
@@ -35,6 +21,11 @@ class InvalidOptionError(IOError):
         super(InvalidOptionError, self).__init__(*args)
 
 
+class InvalidOperationError(IOError):
+    def __init__(self, *args):
+        super(InvalidOperationError, self).__init__(*args)
+
+
 class ArgumentValueError(IOError):
     def __init__(self, *args):
         super(ArgumentValueError, self).__init__(*args)
